
//		Copyright © 1999		Matthew S. Chartier (mattc@ic.net)
//
//		This program is free software; you can redistribute it and/or modify
//		it under the terms of the GNU General Public License as published by
//		the Free Software Foundation; either version 1, or (at your option)
//		any later version.
//
//		This program is distributed in the hope that it will be useful,
//		but WITHOUT ANY WARRANTY; without even the implied warranty of
//		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
//		GNU General Public License for more details.
//
//		You should have received a copy of the GNU General Public License
//		along with this program; if not, write to the Free Software
//		Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//		Portions of this file Copyright Grant R. Guenther (grant@torque.net)
//										David Cambell (cambell@torque.net)		


#include <stdio.h>
#include "parport.h"
#include "ppa_lowlevel.h"

#define imm_pb_release(x)

/*----------------------------------------------------------*/
static void udelay(bigtime_t delay)
{
	snooze(delay);
}


/*----------------------------------------------------------*/
void outsl(int addr,long* buffer,int len)
{
	int l;
	for(l = 0;l < len;l++,buffer++)
		write_io_32(addr,*buffer);
}

/*----------------------------------------------------------*/
void outsb(int addr,char* buffer,int len)
{
	int l;
	for(l = 0;l < len;l++,buffer++)
		write_io_8(addr,*buffer);
}

/*----------------------------------------------------------*/
void insl(int addr,long* buffer,int len)
{
	int l;
	for(l = 0;l < len;l++,buffer++)
		*buffer = read_io_32(addr);
}

/*----------------------------------------------------------*/
void insb(int addr,char* buffer,int len)
{
	int l;
	for(l = 0;l < len;l++,buffer++)
		*buffer = read_io_8(addr);
}


/*----------------------------------------------------------*/
static void ppa_d_pulse(struct parport* port,unsigned char b)
{
	w_dtr(port->base,b);
	w_ctr(port->base,0xc);
	w_ctr(port->base,0xe);
	w_ctr(port->base,0xc);
	w_ctr(port->base,0x4);
	w_ctr(port->base,0xc);
}

/*----------------------------------------------------------*/
static void ppa_c_pulse(struct parport* port,unsigned char b)
{
	w_dtr(port->base,b);
	w_ctr(port->base,0x4);
	w_ctr(port->base,0x6);
	w_ctr(port->base,0x4);
	w_ctr(port->base,0xc);
}

/*----------------------------------------------------------*/
static void ppa_reset_pulse(struct parport* port)
{
	w_ctr(port->base, 0x04);
    w_dtr(port->base, 0x40);
    udelay(1);
    w_ctr(port->base, 0x0c);
    w_ctr(port->base, 0x0d);
    udelay(50);
    w_ctr(port->base, 0x0c);
    w_ctr(port->base, 0x04);
    udelay(5000);
}

/*----------------------------------------------------------*/
static int ppa_select(struct parport* port,int target)
	{
	int k;
	
	w_ctr(port->base, 0xc);
	k = PPA_SELECT_TMO;
	do {
	k--;
	} while((r_str(port->base) & 0x08) && (k));
	
	if(!k) {
	dprintf("imm: imm_select() called while a device selected.\n");
		return(0);
	}
	
	/*
     * Now assert the SCSI ID (HOST and TARGET) on the data bus
     */
    w_ctr(port->base, 0x4);
    w_dtr(port->base, 0x80 | (1 << target));
    udelay(1);

    /*
     * Deassert SELIN first followed by STROBE
     */
    w_ctr(port->base, 0xc);
    w_ctr(port->base, 0xd);

    /*
     * ACK should drop low while SELIN is deasserted.
     * FAULT should drop low when the SCSI device latches the bus.
     */
    k = PPA_SELECT_TMO;
    do {
	k--;
    }
    while (!(r_str(port->base) & 0x08) && (k));

    /*
     * Place the interface back into a sane state (status mode)
     */
    w_ctr(port->base, 0xc);
    return (k) ? 1 : 0;
}
	

/*----------------------------------------------------------*/
static void epp_reset(struct parport* port)
{
	int i;
	
	i = r_str(port->base);
	w_str(port->base,i);
	w_str(port->base,i & 0xfe);
}	

/*----------------------------------------------------------*/
static void ecp_sync(struct parport* port)
{
	int i;
	
	if((r_ecr(port->base) & 0xe0) != 0x80)
		return;
		
	for(i = 0; i < 100; i++) {
		if(r_ecr(port->base) & 0x01)
			return;
		udelay(5);
	}
//#ifdef __USERSPACE
	dprintf("ppa: ECP symc failed\n");
//#endif
}
	

/*----------------------------------------------------------*/
static void ppa_disconnect(struct parport* port)
{
	imm_cpp(port->base, 0x30);
}

/*----------------------------------------------------------*/
static int ppa_connect(struct parport* port,int flag)
{
	unsigned short ppb = port->base;
	
	imm_cpp(ppb, 0xe0);		/* Select device 0 in compatible mode */
    imm_cpp(ppb, 0x30);		/* Disconnect all devices */
    return imm_cpp(ppb, 0xe0);
}

/*----------------------------------------------------------*/
static int ppa_wait(struct parport* port)
	{
	int k;
	unsigned char r;
	
	w_ctr(port->base, 0x0c);
	
	k = PPA_SPIN_TMO;
	do
		{
		r = r_str(port->base);
		k--;
		udelay(1);
		}
	while(!(r & 0x80) && (k));
	
	w_ctr(port->base, 0x04);
	
	if(k)
		return(r & 0xb8);

//#ifdef __USERSPACE
	dprintf("ppa: timeout in ppa_wait\n");
//#endif
	return(0);
	}


/*----------------------------------------------------------*/
static int ppa_byte_out(struct parport* port,const char* buffer,int len)
{
	int i;
	
	w_ctr(port->base, 0x4);
	
	for(i = len >> 1; i; i--) {
		w_dtr(port->base,*buffer++);
		w_ctr(port->base, 0x5);	/* Drop STROBE low */
		w_dtr(port->base, *buffer++);
		w_ctr(port->base, 0x0);	/* STROBE high + INIT low */
	}
	w_ctr(port->base, 0x4);
	
	return(1);	
}

static inline int ppa_byte_in(struct parport* port, char *buffer, int len)
{
    int i;

    /*
     * The following is based on documented timing signals
     */
    w_ctr(port->base, 0x4);
    for (i = len; i; i--) {
	w_ctr(port->base, 0x26);
	*buffer++ = r_dtr(port->base);
	w_ctr(port->base, 0x25);
    }
    return 1;			/* All went well - we hope! */
}
/*----------------------------------------------------------*/
static int ppa_nibble_in(struct parport* port,char* buffer,int len)
{
	unsigned char h, l;
    int i;

    /*
     * The following is based on documented timing signals
     */
    w_ctr(port->base, 0x4);
    for (i = len; i; i--) {
	w_ctr(port->base, 0x6);
	l = r_str(port->base);
	w_ctr(port->base, 0x5);
	h = r_str(port->base);
	w_ctr(port->base, 0x4);
	*buffer++ = (h & 0xf0) | ((l & 0xf0) >> 4);
    }
    return 1;			/* All went well - we hope! */
}

/*----------------------------------------------------------*/
static int ppa_out(struct parport* port,char* buffer,int len)
	{
	int r;
	r = ppa_wait(port);
	
	if((r & 0x18) != 0x08) {
		dprintf("IMM: returned SCSI status %2x\n", r);
		return(0);
	}
	
	//switch(port->modes) {
//		case PPA_EPP_32:
//    	case PPA_EPP_16:
//    	case PPA_EPP_8:
    	//	epp_reset(port);
			//w_ctr(port->base, 0x4);
		//	r = imm_epp_out(port, buffer, len);
		//	w_ctr(port->base, 0xc);
		//	ecp_sync(port);
//		break;
//		case PPA_NIBBLE:
//		case PPA_PS2:
			r = ppa_byte_out(port,buffer,len);
		//break;
		//default:
		//	dprintf("IMM: bug in imm_out()\n");
		//	r = 0;
  //  }
		//r = ppa_byte_out(port,buffer,len);
	return(r);
	}

static inline int imm_epp_out(struct parport* port, char *buffer, int len)
{
    int i;
    for (i = len; i; i--)
	w_epp(port->base, *buffer++);
    return 1;
}

/*----------------------------------------------------------*/
static int ppa_in(struct parport* port, char *buffer, int len)
{
    int r;

    r = ppa_wait(port);

    /*
     * Make sure that:
     * a) the SCSI bus is BUSY (device still listening)
     * b) the device is sending data
     */
    if ((r & 0x18) != 0x18) {
	return 0;
    }
    
    //switch (port->modes) {
    //case PPA_NIBBLE:
	/* 4 bit input, with a loop */
	r = ppa_nibble_in(port, buffer, len);
	w_ctr(port->base, 0xc);
	//break;

   // case PPA_PS2:
	///* 8 bit input, with a loop */
	//r = ppa_byte_in(port, buffer, len);
	//w_ctr(port->base, 0xc);
	//break;

    //case PPA_EPP_32:
//    case PPA_EPP_16:
//    case PPA_EPP_8:
	//epp_reset(port);
	//w_ctr(port->base, 0x24);
	//r = ppa_epp_in(port, buffer, len);
	//w_ctr(port->base, 0x2c);
	//ecp_sync(port);
//	break;
//
//    default:
//	dprintf("IMM: bug in imm_ins()\n");
//	r = 0;
//	break;
//    }
    return r;
}

static inline int ppa_epp_in(struct parport* port, char *buffer, int len)
{
    int i;
    for (i = len; i; i--)
	*buffer++ = r_epp(port->base);
    return 1;
}
//static int ppa_in(struct parport* port,char* buffer,int len)
//	{
//	int r = ppa_wait(port);
//	
//	if((r & 0x18) != 0x18)
//		return(0);
//		r = ppa_nibble_in(port,buffer,len);
//		w_ctr(port->base,0xc);
//	
//	return(r);
//	}


/*----------------------------------------------------------*/
static int device_check(struct parport* port)
{
    if (scsi_com_check(port) == 1)
	return 0;
    return 1;
}

static int scsi_com_check(struct parport* port)
{
    int id, s;
    int len = 6;
    //struct Scsi_Host hst;
    Scsi_Cmnd cmd;

	cmd.cmnd = (char *)malloc(len+1);
    /*
     * The following initalisations can not be performed at
     * declaration. Both gcc and egcs insert memset statements
     * which *SHOULD NOT* be there...
     */
    //hst.unique_id = host_no;
    //cmd.host = &hst;
    cmd.cmnd[0] = 0x00, cmd.cmnd[1] = 0x00, cmd.cmnd[2] = 0x00;
    cmd.cmnd[3] = 0x00, cmd.cmnd[4] = 0x00, cmd.cmnd[5] = 0x00;
    cmd.cmd_len = 6;

    for (id = 0; id < 7; id++) {
	ppa_connect(port,CONNECT_EPP_MAYBE);
	if (!ppa_select(port, id)) {
	    ppa_disconnect(port);
	    continue;
	}
	//printk(IMM_ID "device at ID %i, Attempting to use %s\n", id,
	      // IMM_MODE_STRING[MODE(host_no)]);

	/* Send command */
	if (!ppa_send_command(port, &cmd)) {
	    ppa_disconnect(port);
	    ppa_connect(port, 0);
	    ppa_reset_pulse(port);
	    udelay(1000);
	    ppa_disconnect(port);
	    udelay(1000);
	    return 0;		/* And the smoke starts rising... */
	}
	/* Check status */
	s = ppa_wait(port);

	ppa_disconnect(port);
	ppa_connect(port, 0);
	ppa_reset_pulse(port);
	udelay(1000);
	ppa_disconnect(port);
	udelay(1000);

	if (s == 0xb8) {
		dprintf("com_check: Yes\n");
	   // printk(IMM_ID "Using %s\n", IMM_MODE_STRING[MODE(host_no)]);
	    return 1;		/* It works!! */
	} else {
	    return 0;		/* Failed */
	}
    }
    //printk(IMM_ID "error - SCSI bus stall for %s\n",
	  // IMM_MODE_STRING[MODE(host_no)]);
    ppa_reset_pulse(port);
    ppa_disconnect(port);
   // }
    return -1;			/* No devices found */
}


/*----------------------------------------------------------*/
static int ppa_send_command(struct parport* port,Scsi_Cmnd* cmd)
	{
	int k;

	for(k=0; k < cmd->cmd_len; k += 2)
		if(!ppa_out(port, &cmd->cmnd[k], 2))
			return(0);
	return(1);
	}


/*----------------------------------------------------------*/
static int ppa_completion(struct parport* port, Scsi_Cmnd* cmd)
	{
	bool needDelay = false;
	unsigned char r, v, *ptr;
	int bulk, status, fast, size;

	//unsigned long start_jiffies = system_time();
	
	dprintf("compl: completion\n");
	v = cmd->cmnd[0];
	bulk = ((v == READ_6) || (v == READ_10) || (v == WRITE_6) || (v == WRITE_10));
	
	size = cmd->request_bufflen;
	ptr = cmd->request_buffer;
	
	w_ctr(port->base, 0x0c);
	r   = (r_str(port->base) & 0xb8);
	
	needDelay = (size >= 1024);
	
	while(r != (unsigned char)0xb8)
		{
		
		//if (system_time() > start_jiffies + 1) {
		//	dprintf("sys_time: Yews\n");
	   /// return 0;
	   // }
	    
		if(((r & 0x88) != 0x88) || (size <= 0))
			return(-1);
	
		//fast = (bulk && (size >= PPA_BURST_SIZE)) ? PPA_BURST_SIZE : 1;
//
//		if(r == (unsigned char)0xc0) {
//		dprintf("r in ppa_out is %d\n", r);
//			status = ppa_out(port,ptr,fast);
//		} else {
//			dprintf("r in ppa_in is %d\n", r);
//			status = ppa_in(port,ptr,fast);
//		}

	if (port->rd == 0) {
	    fast = (bulk && (size >= PPA_BURST_SIZE)) ? PPA_BURST_SIZE : 2;
	    status = ppa_out(port, ptr, fast);
	} else {
	    fast = (bulk && (size >= PPA_BURST_SIZE)) ? PPA_BURST_SIZE : 1;
	    status = ppa_in(port, ptr, fast);
	}

		ptr  += fast;
		size -= fast;
		
		if(!status)
			return(-1);

		if(needDelay)
			udelay(5000);
			
			
		if (size & 0x01) {
		    size++;
		    dprintf("IMM: adjusted buffer for 16 bit transfer\n");
		}
		
		w_ctr(port->base, 0x0c);
		r = (r_str(port->base) & 0xb8);
		if(!(r & 0x80))
			return(0);
		}

	return(1);	
	}


static int imm_negotiate(struct parport * tmp)
{
    /*
     * The following is supposedly the IEEE 1248-1994 negotiate
     * sequence. I have yet to obtain a copy of the above standard
     * so this is a bit of a guess...
     *
     * A fair chunk of this is based on the Linux parport implementation
     * of IEEE 1284.
     *
     * Return 0 if data available
     *        1 if no data available
     */

    unsigned short base = tmp->base;
    unsigned char a, mode = 0x00;

   // switch (tmp->mode) {
    //case IMM_NIBBLE:
	//mode = 0x00;
	//break;
   // case IMM_PS2:
	//mode = 0x01;
	//break;
   // default:
	//return 0;
   // }

    w_ctr(base, 0x04);
    udelay(5);
    w_dtr(base, mode);
    udelay(100);
    w_ctr(base, 0x06);
    udelay(5);
    a = (r_str(base) & 0x20) ? 0 : 1;
    udelay(5);
    w_ctr(base, 0x07);
    udelay(5);
    w_ctr(base, 0x06);

    if (a) {
	dprintf("IMM: IEEE1284 negotiate indicates no data available.\n");
	//imm_fail(tmp->host, DID_ERROR);
    }
    return a;
}


/*----------------------------------------------------------*/
static int ppa_engine(struct parport* port,Scsi_Cmnd* cmd)
	{
	int retv, x,  ret = 0;
	unsigned char l = 0,h = 0;

	dprintf("engine\n");
	dprintf("cmd phasetop: %d\n", cmd->phase);
	
	//if(cmd->status != STATUS_FAILED)
	if(cmd->status == STATUS_FAILED)
	return 0;
		//{
		switch(cmd->phase)
			{
			case 0:			/* Phase 0 - wait for device */
				dprintf("engine: case 0\n");
				//ppa_connect(port, 0);
				cmd->phase++;
				//ret = 1;
				//break;
				
			case 1:			/* Phase 1 - connect */
				//retv = 2;
				ppa_connect(port,CONNECT_EPP_MAYBE);
				cmd->phase++;
				
				//w_ctr(port->base,0xe);
				
				//if((r_str(port->base) & 0x08) == 0x08)
					//retv--;
					
				//w_ctr(port->base,0xc);
				//if((r_str(port->base) & 0x08) == 0x00)
					//retv--;
					
				//if(retv)
					//cmd->status = STATUS_FAILED;		
				//else
					//{
					//cmd->phase++;
					//ret = 1;
					//}
				//break;
				
			case 2:			/* Phase 2 - now talking to scsi bus */
				if(!ppa_select(port,cmd->target))
					return 0;
					cmd->phase++;
					//cmd->status = STATUS_FAILED;
//				else
//					{
//					cmd->phase++;
//					ret = 1;
//					}
			//	break;
				
			case 3:			/* Phase 3 - ready to accept a command */
				//w_ctr(port->base,0xc);
//				if(!(r_str(port->base) & 0x80))
//					ret = 1;
//				else
//					if(ppa_send_command(port,cmd))
//						{
// 						cmd->phase++;
//						ret = 1;
//						}
				
				//break;
				w_ctr(port->base, 0x0c);
			if (!(r_str(port->base) & 0x80))
	    		return 1;

			if (!ppa_send_command(port, cmd))
	    		return 0;
			cmd->phase++;
				
			case 4:			/* Phase 4 - setup scatter/gather buffers */
				//cmd->buffer = NULL;
				//cmd->this_residual = cmd->request_bufflen;
	    		//cmd->ptr = cmd->request_buffer;
				cmd->phase++;
				//if (cmd->this_residual & 0x01) {
	    		//	cmd->this_residual++;
	    			dprintf("IMM: adjusted buffer for 16 bit transfer\n");
				//}
				
				//ret = 1;
			//	break;
				
			case 5:			/* Phase 5 - data transfer stage */
				w_ctr(port->base,0x0c);
				if(!(r_str(port->base) & 0x80))
					return(1);
					
					x = (r_str(port->base) & 0xb8);
					port->rd = (x & 0x10) ? 1 : 0;
					port->dp = (x & 0x20) ? 0 : 1;
					
				if ((port->dp) && (port->rd))
	    			if (imm_negotiate(port))
					return 0;
				cmd->phase++;	
					
				//else
//					{
//					retv = ppa_completion(port,cmd);
//					ret  = retv == -1 ? 0 : 1;
//					cmd->phase += ret;
//					}
			//	break;
				
			case 6:			/* Phase 6 - read status/message */
				//cmd->result = DID_OK << 16;
//				if(ppa_wait(port) == (unsigned char)0xb8)
//					if(ppa_in(port,&j,1))
//						if(ppa_wait(port) == (unsigned char)0xb8)
//							{
//							ppa_in(port,&h,1);
//							cmd->result = (DID_OK << 16) + (h << 8) + (j & STATUS_MASK);
//							ret = 0;
//							}
//				break;
			w_ctr(port->base, 0x0c);
			if (!(r_str(port->base) & 0x80))
	    		return 1;

			if (port->dp) {
	    		retv = ppa_completion(port, cmd);
	    		if (retv == -1)
				return 0;
	    		if (retv == 0)
				return 1;
			}
			cmd->phase++;
			case 7:
				if ((port->dp) && (port->rd)) {
					w_ctr(port->base, 0x4);
					w_ctr(port->base, 0xc);
					w_ctr(port->base, 0xe);
					w_ctr(port->base, 0x4);
	    		}
				cmd->phase++;
			
			case 8:
			cmd->result = DID_OK << 16;
			if (ppa_wait(port) != (unsigned char) 0xb8) {
	    		//imm_fail(host_no, DID_ERROR);
	   			return 0;
			}
			if (imm_negotiate(port))
	    		return 0;
			if (ppa_in(port, &l, 1)) {	/* read status byte */
	    		/* Check for optional message byte */
	    		if (ppa_wait(port) == (unsigned char) 0xb8)
				ppa_in(port, &h, 1);
	    		cmd->result = (DID_OK << 16) + (l & STATUS_MASK);
			}
				w_ctr(port->base, 0x4);
	    		w_ctr(port->base, 0xc);
	    		w_ctr(port->base, 0xe);
	    		w_ctr(port->base, 0x4);
	    		
			return 0;		/* Finished */
	break;

    default:
	dprintf("imm: Invalid scsi phase\n");
    }
    return 0;
}


/*----------------------------------------------------------*/
int ppa_command(struct parport* port,Scsi_Cmnd* cmd)
	{
	cmd->result = DID_ERROR << 16;
	cmd->status = 0;
	cmd->phase  = 0;

	while(ppa_engine(port,cmd))
		;
				
	if(cmd->phase)
		ppa_disconnect(port);

	dprintf("ppa_command: %d\n", cmd->result);
	return(cmd->result);
	}


/*----------------------------------------------------------*/

static int imm_cpp(unsigned short ppb, unsigned char b)
{
    /*
     * Comments on udelay values refer to the
     * Command Packet Protocol (CPP) timing diagram.
     */

    unsigned char s1, s2, s3;
    w_ctr(ppb, 0x0c);
    udelay(2);			/* 1 usec - infinite */
    w_dtr(ppb, 0xaa);
    udelay(10);			/* 7 usec - infinite */
    w_dtr(ppb, 0x55);
    udelay(10);			/* 7 usec - infinite */
    w_dtr(ppb, 0x00);
    udelay(10);			/* 7 usec - infinite */
    w_dtr(ppb, 0xff);
    udelay(10);			/* 7 usec - infinite */
    s1 = r_str(ppb) & 0xb8;
    w_dtr(ppb, 0x87);
    udelay(10);			/* 7 usec - infinite */
    s2 = r_str(ppb) & 0xb8;
    w_dtr(ppb, 0x78);
    udelay(10);			/* 7 usec - infinite */
    s3 = r_str(ppb) & 0x38;
    /*
     * Values for b are:
     * 0000 00aa    Assign address aa to current device
     * 0010 00aa    Select device aa in EPP Winbond mode
     * 0010 10aa    Select device aa in EPP mode
     * 0011 xxxx    Deselect all devices
     * 0110 00aa    Test device aa
     * 1101 00aa    Select device aa in ECP mode
     * 1110 00aa    Select device aa in Compatible mode
     */
    w_dtr(ppb, b);
    udelay(2);			/* 1 usec - infinite */
    w_ctr(ppb, 0x0c);
    udelay(10);			/* 7 usec - infinite */
    w_ctr(ppb, 0x0d);
    udelay(2);			/* 1 usec - infinite */
    w_ctr(ppb, 0x0c);
    udelay(10);			/* 7 usec - infinite */
    w_dtr(ppb, 0xff);
    udelay(10);			/* 7 usec - infinite */

    /*
     * The following table is electrical pin values.
     * (BSY is inverted at the CTR register)
     *
     *       BSY  ACK  POut SEL  Fault
     * S1    0    X    1    1    1
     * S2    1    X    0    1    1
     * S3    L    X    1    1    S
     *
     * L => Last device in chain
     * S => Selected
     *
     * Observered values for S1,S2,S3 are:
     * Disconnect => f8/58/78
     * Connect    => f8/58/70
     */
    if ((s1 == 0xb8) && (s2 == 0x18) && (s3 == 0x30)) {
    dprintf("imm_cpp: Yes\n");
	return 1;		/* Connected */
	}
    if ((s1 == 0xb8) && (s2 == 0x18) && (s3 == 0x38)) {
    dprintf("imm_cpp: No\n");
	return 0;		/* Disconnected */
	}
    return -1;			/* No device present */
}

int ppa_init(struct parport* port)
{
	int retv;
	ppa_disconnect(port);
	retv = ppa_connect(port, 0);
	
	if(retv == 1) {
		ppa_reset_pulse(port);
		udelay(1000);
		ppa_disconnect(port);
		udelay(1000);		/* Delay to allow devices to settle */
		ppa_disconnect(port);
		udelay(1000);
		return device_check(port);
	}
	
	return(1);
}
	//int retv;
//	
//	ppa_disconnect(port);
//	ppa_connect(port,CONNECT_NORMAL);	
//	
//	retv = 2;
//	
//	w_ctr(port->base,0xe);
//	if((r_str(port->base) & 0x08) == 0x08)
//		retv--;
//		
//	w_ctr(port->base,0xc);
//	if((r_str(port->base) & 0x08) == 0x00)
//		retv--;
//		
//	if(!retv)
//		ppa_reset_pulse(port);
//	
//	udelay(1000);
//	ppa_disconnect(port);
//	udelay(1000);
//	
//	dprintf("init\n");
//	if(!retv)
//		retv = device_check(port);
//		
//	return(retv);


	
/*----------------------------------------------------------*/
int ppa_SendCommand(struct parport* port, void* cmd, uint cmdlen, void* data, uint datalen)
{
   	Scsi_Cmnd internalCmd;
	dprintf("command\n");
   	memset(&internalCmd,0,sizeof(internalCmd));

   	internalCmd.target          = 6;
   	internalCmd.cmnd            = (char*)cmd;
   	internalCmd.cmd_len         = cmdlen;
   	internalCmd.request_buffer  = (char*)data;
   	internalCmd.request_bufflen = datalen;

   	return(ppa_command(port,&internalCmd));
}

